@@ -12,6 +12,7 @@
 
 import datetime
 import os
+import re
 import subprocess
 import sys
 import time
@@ -73,7 +74,7 @@ def muse(field):
 # =====================================================================
 
 
-@unittest.skipUnless(BSD, "not a BSD system")
+@unittest.skipUnless(BSD, "BSD only")
 class BSDSpecificTestCase(unittest.TestCase):
     """Generic tests common to all BSD variants."""
 
@@ -124,21 +125,34 @@ class BSDSpecificTestCase(unittest.TestCase):
             if abs(usage.used - used) > 10 * 1024 * 1024:
                 self.fail("psutil=%s, df=%s" % (usage.used, used))
 
+    @unittest.skipIf(not which('sysctl'), "sysctl cmd not available")
     def test_cpu_count_logical(self):
         syst = sysctl("hw.ncpu")
         self.assertEqual(psutil.cpu_count(logical=True), syst)
 
+    @unittest.skipIf(not which('sysctl'), "sysctl cmd not available")
     def test_virtual_memory_total(self):
         num = sysctl('hw.physmem')
         self.assertEqual(num, psutil.virtual_memory().total)
 
+    def test_net_if_stats(self):
+        for name, stats in psutil.net_if_stats().items():
+            try:
+                out = sh("ifconfig %s" % name)
+            except RuntimeError:
+                pass
+            else:
+                self.assertEqual(stats.isup, 'RUNNING' in out, msg=out)
+                self.assertEqual(stats.mtu,
+                                 int(re.findall('mtu (\d+)', out)[0]))
+
 
 # =====================================================================
 # --- FreeBSD
 # =====================================================================
 
 
-@unittest.skipUnless(FREEBSD, "not a FreeBSD system")
+@unittest.skipUnless(FREEBSD, "FREEBSD only")
 class FreeBSDSpecificTestCase(unittest.TestCase):
 
     @classmethod
@@ -149,15 +163,8 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
     def tearDownClass(cls):
         reap_children()
 
-    def test_boot_time(self):
-        s = sysctl('sysctl kern.boottime')
-        s = s[s.find(" sec = ") + 7:]
-        s = s[:s.find(',')]
-        btime = int(s)
-        self.assertEqual(btime, psutil.boot_time())
-
     @retry_before_failing()
-    def test_memory_maps(self):
+    def test_proc_memory_maps(self):
         out = sh('procstat -v %s' % self.pid)
         maps = psutil.Process(self.pid).memory_maps(grouped=False)
         lines = out.split('\n')[1:]
@@ -171,17 +178,17 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
             if not map.path.startswith('['):
                 self.assertEqual(fields[10], map.path)
 
-    def test_exe(self):
+    def test_proc_exe(self):
         out = sh('procstat -b %s' % self.pid)
         self.assertEqual(psutil.Process(self.pid).exe(),
                          out.split('\n')[1].split()[-1])
 
-    def test_cmdline(self):
+    def test_proc_cmdline(self):
         out = sh('procstat -c %s' % self.pid)
         self.assertEqual(' '.join(psutil.Process(self.pid).cmdline()),
                          ' '.join(out.split('\n')[1].split()[2:]))
 
-    def test_uids_gids(self):
+    def test_proc_uids_gids(self):
         out = sh('procstat -s %s' % self.pid)
         euid, ruid, suid, egid, rgid, sgid = out.split('\n')[1].split()[2:8]
         p = psutil.Process(self.pid)
@@ -194,6 +201,46 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
         self.assertEqual(gids.effective, int(egid))
         self.assertEqual(gids.saved, int(sgid))
 
+    @retry_before_failing()
+    def test_proc_ctx_switches(self):
+        tested = []
+        out = sh('procstat -r %s' % self.pid)
+        p = psutil.Process(self.pid)
+        for line in out.split('\n'):
+            line = line.lower().strip()
+            if ' voluntary context' in line:
+                pstat_value = int(line.split()[-1])
+                psutil_value = p.num_ctx_switches().voluntary
+                self.assertEqual(pstat_value, psutil_value)
+                tested.append(None)
+            elif ' involuntary context' in line:
+                pstat_value = int(line.split()[-1])
+                psutil_value = p.num_ctx_switches().involuntary
+                self.assertEqual(pstat_value, psutil_value)
+                tested.append(None)
+        if len(tested) != 2:
+            raise RuntimeError("couldn't find lines match in procstat out")
+
+    @retry_before_failing()
+    def test_proc_cpu_times(self):
+        tested = []
+        out = sh('procstat -r %s' % self.pid)
+        p = psutil.Process(self.pid)
+        for line in out.split('\n'):
+            line = line.lower().strip()
+            if 'user time' in line:
+                pstat_value = float('0.' + line.split()[-1].split('.')[-1])
+                psutil_value = p.cpu_times().user
+                self.assertEqual(pstat_value, psutil_value)
+                tested.append(None)
+            elif 'system time' in line:
+                pstat_value = float('0.' + line.split()[-1].split('.')[-1])
+                psutil_value = p.cpu_times().system
+                self.assertEqual(pstat_value, psutil_value)
+                tested.append(None)
+        if len(tested) != 2:
+            raise RuntimeError("couldn't find lines match in procstat out")
+
     # --- virtual_memory(); tests against sysctl
 
     @retry_before_failing()
@@ -234,47 +281,47 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
 
     # --- virtual_memory(); tests against muse
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     def test_muse_vmem_total(self):
         num = muse('Total')
         self.assertEqual(psutil.virtual_memory().total, num)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     @retry_before_failing()
     def test_muse_vmem_active(self):
         num = muse('Active')
         self.assertAlmostEqual(psutil.virtual_memory().active, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     @retry_before_failing()
     def test_muse_vmem_inactive(self):
         num = muse('Inactive')
         self.assertAlmostEqual(psutil.virtual_memory().inactive, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     @retry_before_failing()
     def test_muse_vmem_wired(self):
         num = muse('Wired')
         self.assertAlmostEqual(psutil.virtual_memory().wired, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     @retry_before_failing()
     def test_muse_vmem_cached(self):
         num = muse('Cache')
         self.assertAlmostEqual(psutil.virtual_memory().cached, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     @retry_before_failing()
     def test_muse_vmem_free(self):
         num = muse('Free')
         self.assertAlmostEqual(psutil.virtual_memory().free, num,
                                delta=MEMORY_TOLERANCE)
 
-    @unittest.skipUnless(MUSE_AVAILABLE, "muse cmdline tool is not available")
+    @unittest.skipUnless(MUSE_AVAILABLE, "muse not installed")
     @retry_before_failing()
     def test_muse_vmem_buffers(self):
         num = muse('Buffer')
@@ -301,12 +348,22 @@ class FreeBSDSpecificTestCase(unittest.TestCase):
     #    self.assertAlmostEqual(psutil.cpu_stats().traps,
     #                           sysctl('vm.stats.sys.v_trap'), delta=1000)
 
+    # --- others
+
+    def test_boot_time(self):
+        s = sysctl('sysctl kern.boottime')
+        s = s[s.find(" sec = ") + 7:]
+        s = s[:s.find(',')]
+        btime = int(s)
+        self.assertEqual(btime, psutil.boot_time())
+
 
 # =====================================================================
 # --- OpenBSD
 # =====================================================================
 
-@unittest.skipUnless(OPENBSD, "not an OpenBSD system")
+
+@unittest.skipUnless(OPENBSD, "OPENBSD only")
 class OpenBSDSpecificTestCase(unittest.TestCase):
 
     def test_boot_time(self):
@@ -320,7 +377,8 @@ class OpenBSDSpecificTestCase(unittest.TestCase):
 # --- NetBSD
 # =====================================================================
 
-@unittest.skipUnless(NETBSD, "not a NetBSD system")
+
+@unittest.skipUnless(NETBSD, "NETBSD only")
 class NetBSDSpecificTestCase(unittest.TestCase):
 
     def parse_meminfo(self, look_for):
@@ -330,31 +388,38 @@ class NetBSDSpecificTestCase(unittest.TestCase):
                     return int(line.split()[1]) * 1024
         raise ValueError("can't find %s" % look_for)
 
-    # XXX - failing tests
+    def test_vmem_total(self):
+        self.assertEqual(
+            psutil.virtual_memory().total, self.parse_meminfo("MemTotal:"))
 
-    # def test_vmem_total(self):
-    #     self.assertEqual(
-    #         psutil.virtual_memory().total, self.parse_meminfo("MemTotal:"))
-
-    # def test_vmem_free(self):
-    #     self.assertEqual(
-    #         psutil.virtual_memory().buffers, self.parse_meminfo("MemFree:"))
+    def test_vmem_free(self):
+        self.assertAlmostEqual(
+            psutil.virtual_memory().free, self.parse_meminfo("MemFree:"),
+            delta=MEMORY_TOLERANCE)
 
     def test_vmem_buffers(self):
-        self.assertEqual(
-            psutil.virtual_memory().buffers, self.parse_meminfo("Buffers:"))
+        self.assertAlmostEqual(
+            psutil.virtual_memory().buffers, self.parse_meminfo("Buffers:"),
+            delta=MEMORY_TOLERANCE)
 
     def test_vmem_shared(self):
-        self.assertEqual(
-            psutil.virtual_memory().shared, self.parse_meminfo("MemShared:"))
+        self.assertAlmostEqual(
+            psutil.virtual_memory().shared, self.parse_meminfo("MemShared:"),
+            delta=MEMORY_TOLERANCE)
 
     def test_swapmem_total(self):
-        self.assertEqual(
-            psutil.swap_memory().total, self.parse_meminfo("SwapTotal:"))
+        self.assertAlmostEqual(
+            psutil.swap_memory().total, self.parse_meminfo("SwapTotal:"),
+            delta=MEMORY_TOLERANCE)
 
     def test_swapmem_free(self):
-        self.assertEqual(
-            psutil.swap_memory().free, self.parse_meminfo("SwapFree:"))
+        self.assertAlmostEqual(
+            psutil.swap_memory().free, self.parse_meminfo("SwapFree:"),
+            delta=MEMORY_TOLERANCE)
+
+    def test_swapmem_used(self):
+        smem = psutil.swap_memory()
+        self.assertEqual(smem.used, smem.total - smem.free)
 
     def test_cpu_stats_interrupts(self):
         with open('/proc/stat', 'rb') as f:
